package com.heyang;

import java.util.List;

public class Evaluator {
	private List<Token> tokens;
	private int tokenIdx;
	
	public Evaluator(List<Token> tokens) {
		this.tokens=tokens;
		this.tokenIdx=0;
	}
	
	public double getResult() throws AnalyzerExp{
		return parse_expression();
	}
	
	private double parse_expression() throws AnalyzerExp{
		double left,right;
		Token currentToken;
		
		left=parse_term();
		for(;;) {
			currentToken=fetchToken();
			if(currentToken==null) {
				return left;
			}
			
			if(currentToken.getType()!=Token.TYPE_PLUS && currentToken.getType()!=Token.TYPE_MINUS) {
				returnToken();
				break;
			}
			
			right=parse_term();
			
			if(currentToken.getType()==Token.TYPE_PLUS) {
				left+= right;
			}else if(currentToken.getType()==Token.TYPE_MINUS) {
				left-= right;
			}else {
				returnToken();
			}
		}
		
		return left;
	}
	
	private double parse_term() throws AnalyzerExp{
		double left,right;
		Token currentToken;
		
		left=parse_primary_exp();
		for(;;) {
			currentToken=fetchToken();
			if(currentToken==null) {
				return left;
			}
			
			if(currentToken.getType()!=Token.TYPE_MULTI && currentToken.getType()!=Token.TYPE_DIVIDE) {
				returnToken();
				break;
			}
			
			right=parse_primary_exp();
			
			if(currentToken.getType()==Token.TYPE_MULTI) {
				left*= right;
			}else if(currentToken.getType()==Token.TYPE_DIVIDE) {
				left/= right;
			}
		}
		
		return left;
	}
	
	private double parse_primary_exp() throws AnalyzerExp{
		Token token;
		double retval=0.0;
		
		token=fetchToken();
		if(token==null) {
			return 0;
		}
		
		if(token.getType()==Token.TYPE_DIGITAL) {
			retval= token.getValue();
			return retval;
		}else if(token.getType()==Token.TYPE_LEFT_PAREN){
			retval=parse_expression();
			
			token=fetchToken();
			if(token==null) {
				return retval;
			}
			
			if(token.getType()!=Token.TYPE_RIGHT_PAREN) {
				throw new AnalyzerExp("missing )");
			}
			
			return retval;
		}else {
			throw new AnalyzerExp(token+" should be a digital.");
		}
	}
	
	private Token fetchToken() {
		if(tokenIdx>=tokens.size()) {
			return null;
		}else {
			Token t=tokens.get(tokenIdx);
			tokenIdx++;
			return t;
		}		
	}
	
	private void returnToken() {
		if(tokenIdx>0) {
			tokenIdx--;
		}
	}
}
