package com.heyang;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class Lexer {
	private List<Token> tokens;
	
	public Lexer(String mathExp) throws AnalyzerExp{
		String swallowed="";
		
		tokens=new ArrayList<Token>();
		for(int i=0;i<mathExp.length();i++){
            char c=mathExp.charAt(i);
            
            if(Character.isWhitespace(c)){
            	swallowed="";
            }else if(c=='+'){
            	addText2Tokens(swallowed);
            	swallowed="";
            	tokens.add(new Token(c,Token.TYPE_PLUS));
            }else if(c=='-'){
            	addText2Tokens(swallowed);
            	swallowed="";
            	tokens.add(new Token(c,Token.TYPE_MINUS));
            }else if(c=='*'){
            	addText2Tokens(swallowed);
            	swallowed="";
            	tokens.add(new Token(c,Token.TYPE_MULTI));
            }else if(c=='/'){
            	addText2Tokens(swallowed);
            	swallowed="";
            	tokens.add(new Token(c,Token.TYPE_DIVIDE));
            }else if(c=='('){
            	addText2Tokens(swallowed);
            	swallowed="";
            	tokens.add(new Token(c,Token.TYPE_LEFT_PAREN));
            }else if(c==')'){
            	addText2Tokens(swallowed);
            	swallowed="";
            	tokens.add(new Token(c,Token.TYPE_RIGHT_PAREN));
            }else {
            	swallowed+=c;
            }
		}
		
		if(swallowed.length()>0) {
			tokens.add(new Token(swallowed,Token.TYPE_DIGITAL));
		}
	}
	
	private void addText2Tokens(String text) throws AnalyzerExp{
		if(text.trim().length()>0) {
			tokens.add(new Token(text,Token.TYPE_DIGITAL));
		}
	}
	
	private static String createRepeatedStr(String seed, int n) {
		return String.join("", Collections.nCopies(n, seed));
	}
	
	public void printTokenList() {
		final String layout = "%-20s %-20s %-20s %-20s %s";
		StringBuilder sb = new StringBuilder();

		sb.append("Token list\n");
		sb.append(String.format(layout, "Index", "Type No","Text","Type Desc","\n"));
		final String continuousStar = createRepeatedStr("-", 84);
		sb.append(continuousStar + "\n");
		int index=0;
		for(Token token:tokens) {
			sb.append(String.format(layout, String.valueOf(index),String.valueOf(token.getType()), token.getText(),token.getTypeStr(),"\n"));
			index++;
		}
		
		System.out.println(sb.toString());
	}
	
	public List<Token> getTokens(){
		return tokens;
	}
	
	public static void main(String[] args) throws Exception{
		Lexer lexer=new Lexer("3+(2*5)");
		List<Token> tokens=lexer.getTokens();
		for(Token t:tokens) {
			System.out.print(t);
		}
	}
}
